@@ -6,6 +6,7 @@ Created on Mon Apr  6 15:15:29 2020
 @author: ele
 """
 
+import re
 import logging 
 import json
 import shutil
@@ -13,14 +14,56 @@ import glob
 import pickle
 import itertools
 
-logger = logging.getLogger(__name__)
-logging.basicConfig(format = '%(asctime)s : %(levelname)s : %(module)s: %(message)s', level = 'INFO')
-
-
 class IOUtils:
     
-    @staticmethod
-    def chunkize(iterable, size=10):
+    logger = logging.getLogger(__name__)
+    
+    
+    @classmethod
+    def _expand_user(cls,path):
+        """
+        Expand ~ and ~user constructions.  If user or $HOME is unknown, do nothing.
+        """
+        
+        return glob.os.path.expanduser(path)
+    
+    @classmethod
+    def _rm_trailing_slash(cls,path):
+        """
+        Remove ending path separator if present
+        """
+        if path[-1] == glob.os.sep:
+            path = path[:-1]
+        
+        return path
+    
+    @classmethod
+    def natural_sort(cls,items):
+        """
+        Correctly sort strings in list.  
+        See: http://nedbatchelder.com/blog/200712/human_sorting.html
+        
+        >>> IOUtils.natural_sort(['test10','test1','test12','test2','test22'])
+        ['test1', 'test2', 'test10', 'test12', 'test22']
+        """
+    
+        def alphanum_key(s):
+            """ Turn a string into a list of string and number chunks.
+                "z23a" -> ["z", 23, "a"]
+            """
+            def tryint(s):
+                try:
+                    return int(s)
+                except ValueError:
+                    return s
+                
+            return [ tryint(c) for c in re.split('([0-9]+)', s) ] 
+            
+        return sorted(items, key = alphanum_key)
+        
+    
+    @classmethod
+    def chunkize(cls, iterable, size=10):
         """
         Split iterable into chunks of size `size`.
         
@@ -36,35 +79,41 @@ class IOUtils:
             yield itertools.chain([first], itertools.islice(iterator, size - 1))
 
     
-    @staticmethod
-    def mkdir(path):
+    @classmethod
+    def mkdir(cls,path):
         """
         Create directory (with subdirectories). Equivalent to bash `mkdir -p`.
         
         args:
             path (str) : system path
         """
+        
+        path = cls._expand_user(path)
+        
         if not glob.os.path.exists(path):
             glob.os.makedirs(path)
-            logger.info("Created `{}`".format(path))
+            cls.logger.debug("Created `{}`".format(path))
 
-    @staticmethod
-    def rm(path):
+    @classmethod
+    def rm(cls,path):
         """
         Remove directory/file. Equivalent to bash `rm -r`.
         
         args:
             path (str) : system path
         """
+        
+        path = cls._expand_user(path)
+        
         if glob.os.path.exists(path):
             if glob.os.path.isdir(path):
                 shutil.rmtree(path)
             else:
                 glob.os.remove(path)
-            logger.info("Deleted `{}`".format(path))
+            cls.logger.debug("Deleted `{}`".format(path))
     
-    @staticmethod
-    def exists(path):
+    @classmethod
+    def exists(cls,path):
         """
         Check if directory/file exists.
         
@@ -78,12 +127,14 @@ class IOUtils:
         True
         """
         
+        path = cls._expand_user(path)
+        
         res = glob.os.path.exists(path)
         
         return res
     
-    @staticmethod
-    def join_paths(paths):
+    @classmethod
+    def join_paths(cls,paths):
         """
         Create system paths from elements.
         
@@ -100,8 +151,8 @@ class IOUtils:
         path = glob.os.path.join(*paths)
         return path
     
-    @staticmethod
-    def dname(path):
+    @classmethod
+    def dname(cls,path):
         """
         Return name of main directory.
         
@@ -115,15 +166,12 @@ class IOUtils:
         '/path/to'
         """
         
-        if path[-1] == glob.os.sep:
-            path = path[:-1]
-        
         name = glob.os.path.dirname(path)
         
         return name
     
-    @staticmethod
-    def fname(path):
+    @classmethod
+    def fname(cls,path):
         """
         Return name of file in system path.
         
@@ -136,31 +184,13 @@ class IOUtils:
         >>> IOUtils.fname('/path/to/file.txt')
         'file.txt'
         """
-        
-        if path[-1] == glob.os.sep:
-            path = path[:-1]
+        path = cls._rm_trailing_slash(path)
             
         name = glob.os.path.basename(path)
         return name
-
-    @staticmethod
-    def readf(path,encoding = None):
-        """
-        Return content of a file.
-        
-         args:
-            path (str) : system path
-            encoding (str) : file encondigs
-        
-        return:
-            content (str) : file content
-        """
-        with open(path,encoding = encoding) as infile:
-            content = infile.read()
-        return content
     
-    @staticmethod
-    def load_json(path):
+    @classmethod
+    def load_json(cls,path):
         """
         Load JSON file into dict.
         
@@ -170,12 +200,15 @@ class IOUtils:
         return:
             json_file (dict) : file content
         """
+        
+        path = cls._expand_user(path)
+        
         with open(str(path)) as infile:
             json_file = json.load(infile)
         return json_file
     
-    @staticmethod
-    def save_json(path,item):
+    @classmethod
+    def save_json(cls,path,item):
         """
         Save dict into JSON file .
         
@@ -186,13 +219,14 @@ class IOUtils:
         return:
             json_file (dict) : file content
         """
+        
+        path = cls._expand_user(path)
+        
         with open(str(path),'w') as outfile:
             json.dump(item, outfile)
             
-        logger.info("Saved JSON `{}`".format(path))
-
-    @staticmethod
-    def load_pickle(path):
+    @classmethod
+    def load_pickle(cls,path):
         """
         Load pickled python object.
         
@@ -202,12 +236,15 @@ class IOUtils:
         return:
             item (object) : python object
         """
+        
+        path = cls._expand_user(path)
+        
         with open(str(path), mode = "rb") as infile:
             item = pickle.load(infile)
         return item
       
-    @staticmethod
-    def save_pickle(item,path):
+    @classmethod
+    def save_pickle(cls,item,path):
         """
         Save python object to pickle file .
         
@@ -219,25 +256,48 @@ class IOUtils:
             json_file (dict) : file content
         """
         
+        path = cls._expand_user(path)
+        
         with open(str(path), mode = "wb") as outfile:
-            pickle.dump(item, outfile)
-        logger.info("Saved pickle `{}`".format(path))
-    
-    @staticmethod
-    def ioglob(path, ext = None, recursive = False):
+            pickle.dump(item, outfile)        
+        
+    @classmethod
+    def ioglob(cls, path, ext = None, recursive = False):
         """
-        Iterator yielding the SORTED paths matching a pathname pattern.
-        If the extension is not provided all files are returned.  
+        Backward compatibility.
+        """
+        
+        for p in cls.iglob(path, ext = ext, recursive = recursive, ordered = True):
+            yield p
+            
+    @classmethod
+    def iuglob(cls, path, ext = None, recursive = False):
+        """
+        Backward compatibility.
+        """
+        
+        for p in cls.iglob(path, ext = ext, recursive = recursive):
+            yield p
+
+    
+    @classmethod
+    def iglob(cls, path, ext = None, recursive = False, ordered = False):
+        """
+        Iterator yielding paths matching a path pattern.
+        If the extension is not provided all files are returned. 
+        
+        Note: if `ordered=True` all paths will be load into memory, otherwise lazy loading is used. 
         
         args:
             path (str) : system path
             ext (str) : file extension
             recursive (bool) : check subfolder
             
-        >>> list(IOUtils.ioglob(path = '.', ext = 'py'))
+        >>> list(IOUtils.iglob(path = '.', ext = 'py', ordered = True))
         ['./__init__.py', './ieaiaio.py', './setup.py']
         """
         
+        path = cls._expand_user(path)
         
         ext = "*.{}".format(ext) if ext is not None else "*"
         
@@ -247,36 +307,13 @@ class IOUtils:
             splits.insert(1,"**")
         
         pregex = glob.os.path.join(*splits)
+                
+        path_gen = cls.natural_sort(glob.iglob(pregex)) if ordered else glob.iglob(pregex)
         
-        for p in sorted(glob.iglob(pregex)):
+        for p in path_gen:
             yield p
     
-    @staticmethod
-    def iuglob(path,ext,recursive = False):
-        """
-        Iterator yielding the UNORDERED paths matching a pathname pattern.
-        If the extension is not provided all files are returned.
-        
-        args:
-            path (str) : system path
-            ext (str) : file extension
-            recursive (bool) : check subfolder
-        """
-        
-        ext = "*.{}".format(ext) if ext is not None else "*"
-        
-        splits = [path,ext] 
-        
-        if recursive:
-            splits.insert(1,"**")
-            
-        pregex = glob.os.path.join(*splits)
-        
-        for p in glob.iglob(pregex):
-            yield p 
-        
-    
-if __name__ == "__main__":
+if __name__ == "__main__": 
     import doctest
     doctest.testmod()
 
